;;; This file is used by Emacs to configure it's code editing modes to correctly
;;; indent cbootimage style source code.

((nil      . ((indent-tabs-mode . t)
              (tab-width        . 8)
              (fill-column      . 80)))

 (c-mode   . ((c-basic-offset . 8)))

 (c++-mode . ((c-basic-offset . 8))))
